# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: platform.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='platform.proto',
  package='mpf.vpe',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0eplatform.proto\x12\x07mpf.vpe\"\x0e\n\x0c\x45mptyRequest\"\x0f\n\rEmptyResponse\"\r\n\x0bQuitRequest\"\x99\x01\n\x0cMachineState\x12M\n\x15initial_switch_states\x18\x01 \x03(\x0b\x32..mpf.vpe.MachineState.InitialSwitchStatesEntry\x1a:\n\x18InitialSwitchStatesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\"O\n\x11SwitchDescription\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x17\n\x0fhardware_number\x18\x02 \x01(\t\x12\x13\n\x0bswitch_type\x18\x03 \x01(\t\"8\n\x0f\x43oilDescription\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x17\n\x0fhardware_number\x18\x02 \x01(\t\"a\n\x10LightDescription\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1f\n\x17hardware_channel_number\x18\x02 \x01(\t\x12\x1e\n\x16hardware_channel_color\x18\x03 \x01(\t\"\x9b\x01\n\x0e\x44mdDescription\x12\x0c\n\x04name\x18\x01 \x01(\t\x12;\n\rcolor_mapping\x18\x02 \x01(\x0e\x32$.mpf.vpe.DmdDescription.ColorMapping\x12\r\n\x05width\x18\x03 \x01(\r\x12\x0e\n\x06height\x18\x04 \x01(\r\"\x1f\n\x0c\x43olorMapping\x12\x06\n\x02\x42W\x10\x00\x12\x07\n\x03RGB\x10\x01\"\xbd\x01\n\x12MachineDescription\x12,\n\x08switches\x18\x01 \x03(\x0b\x32\x1a.mpf.vpe.SwitchDescription\x12\'\n\x05\x63oils\x18\x02 \x03(\x0b\x32\x18.mpf.vpe.CoilDescription\x12)\n\x06lights\x18\x03 \x03(\x0b\x32\x19.mpf.vpe.LightDescription\x12%\n\x04\x64mds\x18\x04 \x03(\x0b\x32\x17.mpf.vpe.DmdDescription\"Q\n\rSwitchChanges\x12\x15\n\rswitch_number\x18\x01 \x01(\t\x12\x14\n\x0cswitch_state\x18\x02 \x01(\x08\x12\x13\n\x0b\x63hange_time\x18\x03 \x01(\x04\"E\n\x12SetDmdFrameRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05\x66rame\x18\x02 \x01(\x0c\x12\x12\n\nbrightness\x18\x03 \x01(\x02\"\xa7\x03\n\x08\x43ommands\x12/\n\nfade_light\x18\x01 \x01(\x0b\x32\x19.mpf.vpe.FadeLightRequestH\x00\x12/\n\npulse_coil\x18\x02 \x01(\x0b\x32\x19.mpf.vpe.PulseCoilRequestH\x00\x12\x31\n\x0b\x65nable_coil\x18\x03 \x01(\x0b\x32\x1a.mpf.vpe.EnableCoilRequestH\x00\x12\x33\n\x0c\x64isable_coil\x18\x04 \x01(\x0b\x32\x1b.mpf.vpe.DisableCoilRequestH\x00\x12H\n\x17\x63onfigure_hardware_rule\x18\x05 \x01(\x0b\x32%.mpf.vpe.ConfigureHardwareRuleRequestH\x00\x12\x42\n\x14remove_hardware_rule\x18\x06 \x01(\x0b\x32\".mpf.vpe.RemoveHardwareRuleRequestH\x00\x12\x38\n\x11\x64md_frame_request\x18\x07 \x01(\x0b\x32\x1b.mpf.vpe.SetDmdFrameRequestH\x00\x42\t\n\x07\x63ommand\"\xa0\x01\n\x10\x46\x61\x64\x65LightRequest\x12\x16\n\x0e\x63ommon_fade_ms\x18\x01 \x01(\r\x12\x34\n\x05\x66\x61\x64\x65s\x18\x02 \x03(\x0b\x32%.mpf.vpe.FadeLightRequest.ChannelFade\x1a>\n\x0b\x43hannelFade\x12\x14\n\x0clight_number\x18\x01 \x01(\t\x12\x19\n\x11target_brightness\x18\x02 \x01(\x02\"N\n\x10PulseCoilRequest\x12\x13\n\x0b\x63oil_number\x18\x01 \x01(\t\x12\x10\n\x08pulse_ms\x18\x02 \x01(\r\x12\x13\n\x0bpulse_power\x18\x03 \x01(\x02\"c\n\x11\x45nableCoilRequest\x12\x13\n\x0b\x63oil_number\x18\x01 \x01(\t\x12\x10\n\x08pulse_ms\x18\x02 \x01(\r\x12\x13\n\x0bpulse_power\x18\x03 \x01(\x02\x12\x12\n\nhold_power\x18\x04 \x01(\x02\")\n\x12\x44isableCoilRequest\x12\x13\n\x0b\x63oil_number\x18\x01 \x01(\t\"\x85\x01\n\x1c\x43onfigureHardwareRuleRequest\x12\x13\n\x0b\x63oil_number\x18\x01 \x01(\t\x12\x15\n\rswitch_number\x18\x02 \x01(\t\x12\x10\n\x08pulse_ms\x18\x03 \x01(\r\x12\x13\n\x0bpulse_power\x18\x04 \x01(\x02\x12\x12\n\nhold_power\x18\x05 \x01(\x02\"G\n\x19RemoveHardwareRuleRequest\x12\x13\n\x0b\x63oil_number\x18\x01 \x01(\t\x12\x15\n\rswitch_number\x18\x02 \x01(\t2\x93\x02\n\x12MpfHardwareService\x12\x33\n\x05Start\x12\x15.mpf.vpe.MachineState\x1a\x11.mpf.vpe.Commands0\x01\x12K\n\x15GetMachineDescription\x12\x15.mpf.vpe.EmptyRequest\x1a\x1b.mpf.vpe.MachineDescription\x12\x45\n\x11SendSwitchChanges\x12\x16.mpf.vpe.SwitchChanges\x1a\x16.mpf.vpe.EmptyResponse(\x01\x12\x34\n\x04Quit\x12\x14.mpf.vpe.QuitRequest\x1a\x16.mpf.vpe.EmptyResponseb\x06proto3'
)



_DMDDESCRIPTION_COLORMAPPING = _descriptor.EnumDescriptor(
  name='ColorMapping',
  full_name='mpf.vpe.DmdDescription.ColorMapping',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BW', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RGB', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=594,
  serialized_end=625,
)
_sym_db.RegisterEnumDescriptor(_DMDDESCRIPTION_COLORMAPPING)


_EMPTYREQUEST = _descriptor.Descriptor(
  name='EmptyRequest',
  full_name='mpf.vpe.EmptyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=27,
  serialized_end=41,
)


_EMPTYRESPONSE = _descriptor.Descriptor(
  name='EmptyResponse',
  full_name='mpf.vpe.EmptyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=58,
)


_QUITREQUEST = _descriptor.Descriptor(
  name='QuitRequest',
  full_name='mpf.vpe.QuitRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=73,
)


_MACHINESTATE_INITIALSWITCHSTATESENTRY = _descriptor.Descriptor(
  name='InitialSwitchStatesEntry',
  full_name='mpf.vpe.MachineState.InitialSwitchStatesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mpf.vpe.MachineState.InitialSwitchStatesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='mpf.vpe.MachineState.InitialSwitchStatesEntry.value', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=171,
  serialized_end=229,
)

_MACHINESTATE = _descriptor.Descriptor(
  name='MachineState',
  full_name='mpf.vpe.MachineState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='initial_switch_states', full_name='mpf.vpe.MachineState.initial_switch_states', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MACHINESTATE_INITIALSWITCHSTATESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=76,
  serialized_end=229,
)


_SWITCHDESCRIPTION = _descriptor.Descriptor(
  name='SwitchDescription',
  full_name='mpf.vpe.SwitchDescription',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mpf.vpe.SwitchDescription.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hardware_number', full_name='mpf.vpe.SwitchDescription.hardware_number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='switch_type', full_name='mpf.vpe.SwitchDescription.switch_type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=231,
  serialized_end=310,
)


_COILDESCRIPTION = _descriptor.Descriptor(
  name='CoilDescription',
  full_name='mpf.vpe.CoilDescription',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mpf.vpe.CoilDescription.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hardware_number', full_name='mpf.vpe.CoilDescription.hardware_number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=312,
  serialized_end=368,
)


_LIGHTDESCRIPTION = _descriptor.Descriptor(
  name='LightDescription',
  full_name='mpf.vpe.LightDescription',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mpf.vpe.LightDescription.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hardware_channel_number', full_name='mpf.vpe.LightDescription.hardware_channel_number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hardware_channel_color', full_name='mpf.vpe.LightDescription.hardware_channel_color', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=370,
  serialized_end=467,
)


_DMDDESCRIPTION = _descriptor.Descriptor(
  name='DmdDescription',
  full_name='mpf.vpe.DmdDescription',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mpf.vpe.DmdDescription.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='color_mapping', full_name='mpf.vpe.DmdDescription.color_mapping', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='width', full_name='mpf.vpe.DmdDescription.width', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='height', full_name='mpf.vpe.DmdDescription.height', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DMDDESCRIPTION_COLORMAPPING,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=470,
  serialized_end=625,
)


_MACHINEDESCRIPTION = _descriptor.Descriptor(
  name='MachineDescription',
  full_name='mpf.vpe.MachineDescription',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='switches', full_name='mpf.vpe.MachineDescription.switches', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='coils', full_name='mpf.vpe.MachineDescription.coils', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lights', full_name='mpf.vpe.MachineDescription.lights', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dmds', full_name='mpf.vpe.MachineDescription.dmds', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=628,
  serialized_end=817,
)


_SWITCHCHANGES = _descriptor.Descriptor(
  name='SwitchChanges',
  full_name='mpf.vpe.SwitchChanges',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='switch_number', full_name='mpf.vpe.SwitchChanges.switch_number', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='switch_state', full_name='mpf.vpe.SwitchChanges.switch_state', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='change_time', full_name='mpf.vpe.SwitchChanges.change_time', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=819,
  serialized_end=900,
)


_SETDMDFRAMEREQUEST = _descriptor.Descriptor(
  name='SetDmdFrameRequest',
  full_name='mpf.vpe.SetDmdFrameRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='mpf.vpe.SetDmdFrameRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='frame', full_name='mpf.vpe.SetDmdFrameRequest.frame', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='brightness', full_name='mpf.vpe.SetDmdFrameRequest.brightness', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=902,
  serialized_end=971,
)


_COMMANDS = _descriptor.Descriptor(
  name='Commands',
  full_name='mpf.vpe.Commands',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fade_light', full_name='mpf.vpe.Commands.fade_light', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pulse_coil', full_name='mpf.vpe.Commands.pulse_coil', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enable_coil', full_name='mpf.vpe.Commands.enable_coil', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='disable_coil', full_name='mpf.vpe.Commands.disable_coil', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='configure_hardware_rule', full_name='mpf.vpe.Commands.configure_hardware_rule', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='remove_hardware_rule', full_name='mpf.vpe.Commands.remove_hardware_rule', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dmd_frame_request', full_name='mpf.vpe.Commands.dmd_frame_request', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='command', full_name='mpf.vpe.Commands.command',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=974,
  serialized_end=1397,
)


_FADELIGHTREQUEST_CHANNELFADE = _descriptor.Descriptor(
  name='ChannelFade',
  full_name='mpf.vpe.FadeLightRequest.ChannelFade',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='light_number', full_name='mpf.vpe.FadeLightRequest.ChannelFade.light_number', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='target_brightness', full_name='mpf.vpe.FadeLightRequest.ChannelFade.target_brightness', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1498,
  serialized_end=1560,
)

_FADELIGHTREQUEST = _descriptor.Descriptor(
  name='FadeLightRequest',
  full_name='mpf.vpe.FadeLightRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_fade_ms', full_name='mpf.vpe.FadeLightRequest.common_fade_ms', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fades', full_name='mpf.vpe.FadeLightRequest.fades', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_FADELIGHTREQUEST_CHANNELFADE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1400,
  serialized_end=1560,
)


_PULSECOILREQUEST = _descriptor.Descriptor(
  name='PulseCoilRequest',
  full_name='mpf.vpe.PulseCoilRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='coil_number', full_name='mpf.vpe.PulseCoilRequest.coil_number', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pulse_ms', full_name='mpf.vpe.PulseCoilRequest.pulse_ms', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pulse_power', full_name='mpf.vpe.PulseCoilRequest.pulse_power', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1562,
  serialized_end=1640,
)


_ENABLECOILREQUEST = _descriptor.Descriptor(
  name='EnableCoilRequest',
  full_name='mpf.vpe.EnableCoilRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='coil_number', full_name='mpf.vpe.EnableCoilRequest.coil_number', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pulse_ms', full_name='mpf.vpe.EnableCoilRequest.pulse_ms', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pulse_power', full_name='mpf.vpe.EnableCoilRequest.pulse_power', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hold_power', full_name='mpf.vpe.EnableCoilRequest.hold_power', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1642,
  serialized_end=1741,
)


_DISABLECOILREQUEST = _descriptor.Descriptor(
  name='DisableCoilRequest',
  full_name='mpf.vpe.DisableCoilRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='coil_number', full_name='mpf.vpe.DisableCoilRequest.coil_number', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1743,
  serialized_end=1784,
)


_CONFIGUREHARDWARERULEREQUEST = _descriptor.Descriptor(
  name='ConfigureHardwareRuleRequest',
  full_name='mpf.vpe.ConfigureHardwareRuleRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='coil_number', full_name='mpf.vpe.ConfigureHardwareRuleRequest.coil_number', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='switch_number', full_name='mpf.vpe.ConfigureHardwareRuleRequest.switch_number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pulse_ms', full_name='mpf.vpe.ConfigureHardwareRuleRequest.pulse_ms', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pulse_power', full_name='mpf.vpe.ConfigureHardwareRuleRequest.pulse_power', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hold_power', full_name='mpf.vpe.ConfigureHardwareRuleRequest.hold_power', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1787,
  serialized_end=1920,
)


_REMOVEHARDWARERULEREQUEST = _descriptor.Descriptor(
  name='RemoveHardwareRuleRequest',
  full_name='mpf.vpe.RemoveHardwareRuleRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='coil_number', full_name='mpf.vpe.RemoveHardwareRuleRequest.coil_number', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='switch_number', full_name='mpf.vpe.RemoveHardwareRuleRequest.switch_number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1922,
  serialized_end=1993,
)

_MACHINESTATE_INITIALSWITCHSTATESENTRY.containing_type = _MACHINESTATE
_MACHINESTATE.fields_by_name['initial_switch_states'].message_type = _MACHINESTATE_INITIALSWITCHSTATESENTRY
_DMDDESCRIPTION.fields_by_name['color_mapping'].enum_type = _DMDDESCRIPTION_COLORMAPPING
_DMDDESCRIPTION_COLORMAPPING.containing_type = _DMDDESCRIPTION
_MACHINEDESCRIPTION.fields_by_name['switches'].message_type = _SWITCHDESCRIPTION
_MACHINEDESCRIPTION.fields_by_name['coils'].message_type = _COILDESCRIPTION
_MACHINEDESCRIPTION.fields_by_name['lights'].message_type = _LIGHTDESCRIPTION
_MACHINEDESCRIPTION.fields_by_name['dmds'].message_type = _DMDDESCRIPTION
_COMMANDS.fields_by_name['fade_light'].message_type = _FADELIGHTREQUEST
_COMMANDS.fields_by_name['pulse_coil'].message_type = _PULSECOILREQUEST
_COMMANDS.fields_by_name['enable_coil'].message_type = _ENABLECOILREQUEST
_COMMANDS.fields_by_name['disable_coil'].message_type = _DISABLECOILREQUEST
_COMMANDS.fields_by_name['configure_hardware_rule'].message_type = _CONFIGUREHARDWARERULEREQUEST
_COMMANDS.fields_by_name['remove_hardware_rule'].message_type = _REMOVEHARDWARERULEREQUEST
_COMMANDS.fields_by_name['dmd_frame_request'].message_type = _SETDMDFRAMEREQUEST
_COMMANDS.oneofs_by_name['command'].fields.append(
  _COMMANDS.fields_by_name['fade_light'])
_COMMANDS.fields_by_name['fade_light'].containing_oneof = _COMMANDS.oneofs_by_name['command']
_COMMANDS.oneofs_by_name['command'].fields.append(
  _COMMANDS.fields_by_name['pulse_coil'])
_COMMANDS.fields_by_name['pulse_coil'].containing_oneof = _COMMANDS.oneofs_by_name['command']
_COMMANDS.oneofs_by_name['command'].fields.append(
  _COMMANDS.fields_by_name['enable_coil'])
_COMMANDS.fields_by_name['enable_coil'].containing_oneof = _COMMANDS.oneofs_by_name['command']
_COMMANDS.oneofs_by_name['command'].fields.append(
  _COMMANDS.fields_by_name['disable_coil'])
_COMMANDS.fields_by_name['disable_coil'].containing_oneof = _COMMANDS.oneofs_by_name['command']
_COMMANDS.oneofs_by_name['command'].fields.append(
  _COMMANDS.fields_by_name['configure_hardware_rule'])
_COMMANDS.fields_by_name['configure_hardware_rule'].containing_oneof = _COMMANDS.oneofs_by_name['command']
_COMMANDS.oneofs_by_name['command'].fields.append(
  _COMMANDS.fields_by_name['remove_hardware_rule'])
_COMMANDS.fields_by_name['remove_hardware_rule'].containing_oneof = _COMMANDS.oneofs_by_name['command']
_COMMANDS.oneofs_by_name['command'].fields.append(
  _COMMANDS.fields_by_name['dmd_frame_request'])
_COMMANDS.fields_by_name['dmd_frame_request'].containing_oneof = _COMMANDS.oneofs_by_name['command']
_FADELIGHTREQUEST_CHANNELFADE.containing_type = _FADELIGHTREQUEST
_FADELIGHTREQUEST.fields_by_name['fades'].message_type = _FADELIGHTREQUEST_CHANNELFADE
DESCRIPTOR.message_types_by_name['EmptyRequest'] = _EMPTYREQUEST
DESCRIPTOR.message_types_by_name['EmptyResponse'] = _EMPTYRESPONSE
DESCRIPTOR.message_types_by_name['QuitRequest'] = _QUITREQUEST
DESCRIPTOR.message_types_by_name['MachineState'] = _MACHINESTATE
DESCRIPTOR.message_types_by_name['SwitchDescription'] = _SWITCHDESCRIPTION
DESCRIPTOR.message_types_by_name['CoilDescription'] = _COILDESCRIPTION
DESCRIPTOR.message_types_by_name['LightDescription'] = _LIGHTDESCRIPTION
DESCRIPTOR.message_types_by_name['DmdDescription'] = _DMDDESCRIPTION
DESCRIPTOR.message_types_by_name['MachineDescription'] = _MACHINEDESCRIPTION
DESCRIPTOR.message_types_by_name['SwitchChanges'] = _SWITCHCHANGES
DESCRIPTOR.message_types_by_name['SetDmdFrameRequest'] = _SETDMDFRAMEREQUEST
DESCRIPTOR.message_types_by_name['Commands'] = _COMMANDS
DESCRIPTOR.message_types_by_name['FadeLightRequest'] = _FADELIGHTREQUEST
DESCRIPTOR.message_types_by_name['PulseCoilRequest'] = _PULSECOILREQUEST
DESCRIPTOR.message_types_by_name['EnableCoilRequest'] = _ENABLECOILREQUEST
DESCRIPTOR.message_types_by_name['DisableCoilRequest'] = _DISABLECOILREQUEST
DESCRIPTOR.message_types_by_name['ConfigureHardwareRuleRequest'] = _CONFIGUREHARDWARERULEREQUEST
DESCRIPTOR.message_types_by_name['RemoveHardwareRuleRequest'] = _REMOVEHARDWARERULEREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EmptyRequest = _reflection.GeneratedProtocolMessageType('EmptyRequest', (_message.Message,), {
  'DESCRIPTOR' : _EMPTYREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.EmptyRequest)
  })
_sym_db.RegisterMessage(EmptyRequest)

EmptyResponse = _reflection.GeneratedProtocolMessageType('EmptyResponse', (_message.Message,), {
  'DESCRIPTOR' : _EMPTYRESPONSE,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.EmptyResponse)
  })
_sym_db.RegisterMessage(EmptyResponse)

QuitRequest = _reflection.GeneratedProtocolMessageType('QuitRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUITREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.QuitRequest)
  })
_sym_db.RegisterMessage(QuitRequest)

MachineState = _reflection.GeneratedProtocolMessageType('MachineState', (_message.Message,), {

  'InitialSwitchStatesEntry' : _reflection.GeneratedProtocolMessageType('InitialSwitchStatesEntry', (_message.Message,), {
    'DESCRIPTOR' : _MACHINESTATE_INITIALSWITCHSTATESENTRY,
    '__module__' : 'platform_pb2'
    # @@protoc_insertion_point(class_scope:mpf.vpe.MachineState.InitialSwitchStatesEntry)
    })
  ,
  'DESCRIPTOR' : _MACHINESTATE,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.MachineState)
  })
_sym_db.RegisterMessage(MachineState)
_sym_db.RegisterMessage(MachineState.InitialSwitchStatesEntry)

SwitchDescription = _reflection.GeneratedProtocolMessageType('SwitchDescription', (_message.Message,), {
  'DESCRIPTOR' : _SWITCHDESCRIPTION,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.SwitchDescription)
  })
_sym_db.RegisterMessage(SwitchDescription)

CoilDescription = _reflection.GeneratedProtocolMessageType('CoilDescription', (_message.Message,), {
  'DESCRIPTOR' : _COILDESCRIPTION,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.CoilDescription)
  })
_sym_db.RegisterMessage(CoilDescription)

LightDescription = _reflection.GeneratedProtocolMessageType('LightDescription', (_message.Message,), {
  'DESCRIPTOR' : _LIGHTDESCRIPTION,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.LightDescription)
  })
_sym_db.RegisterMessage(LightDescription)

DmdDescription = _reflection.GeneratedProtocolMessageType('DmdDescription', (_message.Message,), {
  'DESCRIPTOR' : _DMDDESCRIPTION,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.DmdDescription)
  })
_sym_db.RegisterMessage(DmdDescription)

MachineDescription = _reflection.GeneratedProtocolMessageType('MachineDescription', (_message.Message,), {
  'DESCRIPTOR' : _MACHINEDESCRIPTION,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.MachineDescription)
  })
_sym_db.RegisterMessage(MachineDescription)

SwitchChanges = _reflection.GeneratedProtocolMessageType('SwitchChanges', (_message.Message,), {
  'DESCRIPTOR' : _SWITCHCHANGES,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.SwitchChanges)
  })
_sym_db.RegisterMessage(SwitchChanges)

SetDmdFrameRequest = _reflection.GeneratedProtocolMessageType('SetDmdFrameRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETDMDFRAMEREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.SetDmdFrameRequest)
  })
_sym_db.RegisterMessage(SetDmdFrameRequest)

Commands = _reflection.GeneratedProtocolMessageType('Commands', (_message.Message,), {
  'DESCRIPTOR' : _COMMANDS,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.Commands)
  })
_sym_db.RegisterMessage(Commands)

FadeLightRequest = _reflection.GeneratedProtocolMessageType('FadeLightRequest', (_message.Message,), {

  'ChannelFade' : _reflection.GeneratedProtocolMessageType('ChannelFade', (_message.Message,), {
    'DESCRIPTOR' : _FADELIGHTREQUEST_CHANNELFADE,
    '__module__' : 'platform_pb2'
    # @@protoc_insertion_point(class_scope:mpf.vpe.FadeLightRequest.ChannelFade)
    })
  ,
  'DESCRIPTOR' : _FADELIGHTREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.FadeLightRequest)
  })
_sym_db.RegisterMessage(FadeLightRequest)
_sym_db.RegisterMessage(FadeLightRequest.ChannelFade)

PulseCoilRequest = _reflection.GeneratedProtocolMessageType('PulseCoilRequest', (_message.Message,), {
  'DESCRIPTOR' : _PULSECOILREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.PulseCoilRequest)
  })
_sym_db.RegisterMessage(PulseCoilRequest)

EnableCoilRequest = _reflection.GeneratedProtocolMessageType('EnableCoilRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENABLECOILREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.EnableCoilRequest)
  })
_sym_db.RegisterMessage(EnableCoilRequest)

DisableCoilRequest = _reflection.GeneratedProtocolMessageType('DisableCoilRequest', (_message.Message,), {
  'DESCRIPTOR' : _DISABLECOILREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.DisableCoilRequest)
  })
_sym_db.RegisterMessage(DisableCoilRequest)

ConfigureHardwareRuleRequest = _reflection.GeneratedProtocolMessageType('ConfigureHardwareRuleRequest', (_message.Message,), {
  'DESCRIPTOR' : _CONFIGUREHARDWARERULEREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.ConfigureHardwareRuleRequest)
  })
_sym_db.RegisterMessage(ConfigureHardwareRuleRequest)

RemoveHardwareRuleRequest = _reflection.GeneratedProtocolMessageType('RemoveHardwareRuleRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEHARDWARERULEREQUEST,
  '__module__' : 'platform_pb2'
  # @@protoc_insertion_point(class_scope:mpf.vpe.RemoveHardwareRuleRequest)
  })
_sym_db.RegisterMessage(RemoveHardwareRuleRequest)


_MACHINESTATE_INITIALSWITCHSTATESENTRY._options = None

_MPFHARDWARESERVICE = _descriptor.ServiceDescriptor(
  name='MpfHardwareService',
  full_name='mpf.vpe.MpfHardwareService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1996,
  serialized_end=2271,
  methods=[
  _descriptor.MethodDescriptor(
    name='Start',
    full_name='mpf.vpe.MpfHardwareService.Start',
    index=0,
    containing_service=None,
    input_type=_MACHINESTATE,
    output_type=_COMMANDS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetMachineDescription',
    full_name='mpf.vpe.MpfHardwareService.GetMachineDescription',
    index=1,
    containing_service=None,
    input_type=_EMPTYREQUEST,
    output_type=_MACHINEDESCRIPTION,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SendSwitchChanges',
    full_name='mpf.vpe.MpfHardwareService.SendSwitchChanges',
    index=2,
    containing_service=None,
    input_type=_SWITCHCHANGES,
    output_type=_EMPTYRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Quit',
    full_name='mpf.vpe.MpfHardwareService.Quit',
    index=3,
    containing_service=None,
    input_type=_QUITREQUEST,
    output_type=_EMPTYRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_MPFHARDWARESERVICE)

DESCRIPTOR.services_by_name['MpfHardwareService'] = _MPFHARDWARESERVICE

# @@protoc_insertion_point(module_scope)



from unittest.mock import MagicMock

from mpf.platforms.interfaces.driver_platform_interface import PulseSettings

from mpf.core.platform import SwitchSettings, DriverSettings

from mpf.tests.MpfTestCase import MpfTestCase


class TestKickback(MpfTestCase):

    def get_config_file(self):
        return 'config.yaml'

    def get_machine_path(self):
        return 'tests/machine_files/kickback/'

    def test_kickback_with_ball_save(self):
        self.machine.default_platform.set_pulse_on_hit_rule = MagicMock()
        self.mock_event("kickback_kickback_test_fired")
        self.assertFalse(self.machine.ball_saves["kickback_save"].enabled)

        # kickback is not enabled. nothing should happen
        self.hit_and_release_switch("s_kickback")
        self.advance_time_and_run(.01)
        self.assertEventNotCalled("kickback_kickback_test_fired")

        # enable kickback
        self.post_event("kickback_enable")
        self.advance_time_and_run(.01)

        # should write a hw rule
        self.machine.default_platform.set_pulse_on_hit_rule.assert_called_once_with(
            SwitchSettings(hw_switch=self.machine.switches["s_kickback"].hw_switch, invert=False, debounce=False),
            DriverSettings(hw_driver=self.machine.coils["kickback_coil"].hw_driver,
                           pulse_settings=PulseSettings(power=1.0, duration=100), hold_settings=None, recycle=True)
        )

        # a hit should fire it
        self.hit_and_release_switch("s_kickback")
        self.advance_time_and_run(.01)
        self.assertEventCalled("kickback_kickback_test_fired")

        # ball save should be enabled just in case
        self.assertTrue(self.machine.ball_saves["kickback_save"].enabled)

        # but disable after 6s
        self.advance_time_and_run(6.1)
        self.assertFalse(self.machine.ball_saves["kickback_save"].enabled)

        # it only works once though
        self.mock_event("kickback_kickback_test_fired")
        self.hit_and_release_switch("s_kickback")
        self.advance_time_and_run(.01)
        self.assertEventNotCalled("kickback_kickback_test_fired")



"""In-memory DataManager."""
import copy
from mpf.core.data_manager import DataManager


class TestDataManager(DataManager):

    """A patched version of the DataManager which is used in unit tests.

    The main change is that the ``save_all()`` method doesn't actually
    write anything to disk so the tests don't fill up the disk with
    unneeded data.

    """

    def __init__(self, data):
        self.data = data
        self.written_data = None

    def _trigger_save(self):
        self.written_data = copy.deepcopy(self.data)



"""MPF plugin which uses lights to represent Game functions.

Typically in an EM machine.
"""

import logging


class InfoLights:

    """Uses lights to represent game state.

    Info lights are primarily used in EM and early solid state machines,
    typically lights in the backbox for game over, tilt, which player is up,
    the current ball number, etc.

    """

    def __init__(self, machine):
        """Initialise info lights plugin."""
        self.log = logging.getLogger('infolights')
        self.machine = machine
        self.game_over_show = None

        try:
            self.config = self.machine.config['info_lights']
        except KeyError:
            self.machine.log.debug('"info_lights:" section not found in machine '
                                   'configuration, so the Info Lights plugin '
                                   'will not be used.')
            return

        self.machine.events.add_handler('init_phase_5', self._initialize)

    def _initialize(self, **kwargs):
        del kwargs
        # convert any light names we find to objects
        for key, value in self.config.items():
            if 'light' in value:
                if value['light'] in self.machine.lights:
                    self.config[key]['light'] = self.machine.lights[value['light']]
                else:
                    raise AssertionError("Invalid light {}".format(value['light']))

        self.machine.events.add_handler('ball_started', self._ball_started)
        self.machine.events.add_handler('game_ended', self._game_ended)
        self.machine.events.add_handler('game_starting', self._game_starting)
        self.machine.events.add_handler('player_added', self._player_added)
        self.machine.events.add_handler('tilt', self._tilt)
        self.machine.events.add_handler('match', self._match)

        self._game_ended()  # set the initial lights to the game ended state

    def _reset_game_lights(self):
        self.log.debug("reset_game_lights")
        # turn off the game-specific lights (player, ball & match)
        for key, value in self.config.items():
            if key.startswith('ball_'):
                value['light'].off()
            if key.startswith('player_'):
                value['light'].off()
            if key.startswith('match_'):
                value['light'].off()

    def _ball_started(self, **kwargs):
        del kwargs
        self.log.debug("ball_started")
        # turn off all the ball lights
        for key, value in self.config.items():
            if key.startswith('ball_'):
                value['light'].off()

        # turn on this current ball's light
        ball_light = 'ball_' + str(self.machine.game.player.ball)
        if ball_light in self.config:
            self.config[ball_light]['light'].on()

        # turn off the tilt light
        if 'tilt' in self.config:
            self.config['tilt']['light'].off()

    def _game_ended(self, **kwargs):
        del kwargs
        self.log.debug("game_ended")
        self._reset_game_lights()

        # turn on game over
        if 'game_over' in self.config:
            if self.game_over_show:
                self.game_over_show.stop()
            self.game_over_show = self.machine.shows['flash'].play(
                show_tokens=dict(leds=self.config['game_over']['light']))

    def _game_starting(self, **kwargs):
        del kwargs
        self.log.debug("game_starting")
        self._reset_game_lights()
        if self.game_over_show:
            self.game_over_show.stop()

    def _player_added(self, player, **kwargs):
        del kwargs
        self.log.debug("player_added. player=%s", player)
        player_str = 'player_' + str(player.number)
        self.log.debug("player_str: %s", player_str)
        if player_str in self.config:
            self.config[player_str]['light'].on()

    def _tilt(self, **kwargs):
        del kwargs
        self.log.debug("tilt")
        if 'tilt' in self.config:
            self.config['tilt']['light'].on()

    def _match(self, match, **kwargs):
        del kwargs
        self.log.debug("Match")
        match_str = 'match_' + str(match)
        if match_str in self.config:
            self.config[match_str]['light'].on()



from mpf.tests.MpfBcpTestCase import MpfBcpTestCase
from mpf.tests.MpfTestCase import test_config


class TestDmd(MpfBcpTestCase):

    def get_machine_path(self):
        return 'tests/machine_files/dmd/'

    @test_config("testDmd.yaml")
    def testDmd(self):
        self.machine.dmds["test_dmd"].update(b'12345')
        self.assertEqual(b'12345', self.machine.dmds["test_dmd"].hw_device.data)

        self._bcp_client.receive_queue.put_nowait(("dmd_frame", {"name": "test_dmd", "rawbytes": b'1337'}))
        self.machine_run()

        self.assertEqual(b'1337', self.machine.dmds["test_dmd"].hw_device.data)

    @test_config("testRgbDmd.yaml")
    def testRgbDmd(self):
        self.machine.rgb_dmds["test_dmd"].update(b'12345')
        self.assertEqual(b'12345', self.machine.rgb_dmds["test_dmd"].hw_device.data)

        self._bcp_client.receive_queue.put_nowait(("rgb_dmd_frame", {"name": "test_dmd", "rawbytes": b'1337'}))
        self.machine_run()

        self.assertEqual(b'1337', self.machine.rgb_dmds["test_dmd"].hw_device.data)

        display = self.machine.rgb_dmds["test_dmd"]
        self.assertEqual(1.0, display.hw_device.brightness)

        self.machine.settings.set_setting_value("dmd_brightness", 0.75)
        self.advance_time_and_run()

        self.assertEqual(0.75, display.hw_device.brightness)



"""Mission Pinball Framework (mpf) setup.py."""
import re
from setuptools import setup

#  http://stackoverflow.com/questions/458550/standard-way-to-embed-version-into-python-package
VERSIONFILE = "mpf/_version.py"
VERSION_STRING_LONG = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
_MO = re.search(VSRE, VERSION_STRING_LONG, re.M)
if _MO:
    VERSION_STRING = _MO.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

pin2dmd_requires = [
    'pyusb==1.1.0'
]

linux_i2c_requires = [
    'smbus2_asyncio==0.0.5'
]

rpi_requires = [
    'apigpio-mpf==0.0.3'
]

cli_requires = [
    'prompt_toolkit==3.0.8',
    'asciimatics==1.12.0',
    'terminaltables==3.1.0',
]

osc_requires = [
    'python-osc==1.7.4'
]

irc_requires = [
    'irc==19.0.1'
]

vpe_requires = [
    'grpcio_tools==1.34.0',
    'grpcio==1.34.0',
    'protobuf==3.14.0',
]


all_requires = (pin2dmd_requires + cli_requires + linux_i2c_requires + rpi_requires + osc_requires + irc_requires +
                vpe_requires)

setup(

    name='mpf',
    version=VERSION_STRING,
    description='Mission Pinball Framework',
    long_description='''Let's build a pinball machine!

The Mission Pinball Framework (MPF) is an open source, cross-platform,
Python-based software framework for powering real pinball machines.

MPF is written in Python. It can run on Windows, OS X, and Linux
with the same code and configurations.

MPF interacts with real, physical pinball machines via modern pinball
controller hardware such as a Multimorphic P-ROC or P3-ROC, a FAST Pinball
controller, or Open Pinball Project hardware controllers. You can use MPF to
power your own custom-built machine or to update the software in existing
Williams, Bally, Stern, or Data East machines.

MPF is a work-in-progress that is not yet complete, though we're actively
developing it and checking in several commits a week. It's MIT licensed,
actively developed by fun people, and supported by a vibrant, pinball-loving
community.''',

    url='http://missionpinball.org',
    author='The Mission Pinball Framework Team',
    author_email='brian@missionpinball.org',
    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Topic :: Artistic Software',
        'Topic :: Games/Entertainment :: Arcade'

    ],

    keywords='pinball',

    include_package_data=True,
    package_data={'': ['*.yaml', '*.png', '*.so', '*.pyd', '*.ogg', '*.wav']},

    # MANIFEST.in picks up the rest
    packages=['mpf'],

    zip_safe=False,

    install_requires=['ruamel.yaml==0.15.100',
                      'pyserial==3.5',
                      'pyserial-asyncio==0.4;platform_system=="Windows"',
                      'pyserial-asyncio==0.5;platform_system!="Windows"',
                      'sortedcontainers==2.3.0',
                      'psutil==5.7.3',
                      ],

    extras_require={
        'all': all_requires,
        'pin2dmd': pin2dmd_requires,
        'linux_i2c': linux_i2c_requires,
        'rpi': rpi_requires,
        'cli': cli_requires,
        'osc': osc_requires,
        'irc': irc_requires,
        'vpe': vpe_requires,
    },

    tests_require=[],
    test_suite="mpf.tests",

    entry_points={
        'console_scripts': [
            'mpf = mpf.commands:run_from_command_line',
        ]
    }
)



"""Perform hardware operations."""
import os

from mpf.core.config_loader import YamlMultifileConfigLoader

from mpf.commands import MpfCommandLineParser

from mpf.core.machine import MachineController

SUBCOMMAND = True


class Command(MpfCommandLineParser):

    """Performs hardware operations."""

    def __init__(self, args, path):
        """Parse args."""
        command_name = args.pop(1)
        super().__init__(args=args, path=path)

        machine_path, remaining_args = self.parse_args()
        self.machine_path = machine_path
        self.args = remaining_args
        config_loader = YamlMultifileConfigLoader(machine_path, ["config.yaml"], True, False)

        config = config_loader.load_mpf_config()

        self.mpf = MachineController({"bcp": False,
                                      "no_load_cache": False,
                                      "mpfconfigfile": os.path.join(self.mpf_path, "mpfconfig.yaml"),
                                      "configfile": ["config"],
                                      "production": False,
                                      "create_config_cache": False,
                                      "force_platform": False,
                                      "text_ui": False
                                      }, config)
        self.mpf.clock.loop.run_until_complete(self.mpf.initialise_core_and_hardware())
        if self.mpf.thread_stopper.is_set():
            raise AssertionError("Initialisation failed!")

        method = getattr(self, command_name)
        method()

    def scan(self):
        """Scan hardware."""
        for name, platform in self.mpf.hardware_platforms.items():
            print("{}:".format(name))
            print(platform.get_info_string())
            print("---------")

        self.mpf.shutdown()

    def firmware_update(self):
        """Upgrade firmware of platforms."""
        for name, platform in self.mpf.hardware_platforms.items():
            print("{}:".format(name))
            print(platform.update_firmware())
            print("---------")

        self.mpf.shutdown()



"""Platform interface for servos."""
from typing import List

import abc


class ServoPlatformInterface(metaclass=abc.ABCMeta):

    """Interface for servos in hardware platforms.

    ServoPlatformInterface is an abstract base class that should be overridden for all
    servo interface classes on supported platforms.  This class ensures the proper required
    methods are implemented to support servo operations in MPF.
    """

    __slots__ = []  # type: List[str]

    @abc.abstractmethod
    def go_to_position(self, position):
        """Move servo to a certain position."""
        raise NotImplementedError

    def set_speed_limit(self, speed_limit):
        """Set speed limit."""

    def set_acceleration_limit(self, acceleration_limit):
        """Set acceleration limit."""



from mpf.platforms.interfaces.driver_platform_interface import PulseSettings, HoldSettings

from mpf.core.platform import SwitchSettings, DriverSettings, RepulseSettings

from mpf.tests.MpfTestCase import MpfTestCase
from unittest.mock import MagicMock, call


class TestFlippersSoftwareEosRepulse(MpfTestCase):

    def get_config_file(self):
        return 'software_eos_repulse.yaml'

    def get_machine_path(self):
        return 'tests/machine_files/flippers/'

    def get_platform(self):
        return 'virtual'

    def test_single_wound_flipper_with_software_eos_repulse(self):
        """Test software repulse behaviour.

        Assuming platform supports cut-off
        """
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse = MagicMock()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable = MagicMock()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable = MagicMock()

        # nothing should happen on EOS close without flipper button active
        self.hit_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # nothing should happen on EOS open without flipper button active
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # flipper activation should also not yet do what
        self.hit_switch_and_run("s_flipper_single", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS opens. nothing should happen as flipper is not enabled
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # release flipper prior to enabling the flipper
        self.release_switch_and_run("s_flipper_single", 1)

        self.machine.default_platform.set_pulse_on_hit_and_enable_and_release_and_disable_rule = MagicMock()
        self.assertPlaceholderEvaluates(False, "device.flippers.single_flipper.enabled")

        # enable flipper
        self.post_event("enable_flipper_single")
        self.assertPlaceholderEvaluates(True, "device.flippers.single_flipper.enabled")

        self.assertEqual(1, len(self.machine.default_platform.set_pulse_on_hit_and_enable_and_release_and_disable_rule.
                                _mock_call_args_list))
        self.machine.default_platform.set_pulse_on_hit_and_enable_and_release_and_disable_rule.assert_called_once_with(
            SwitchSettings(hw_switch=self.machine.switches["s_flipper_single"].hw_switch, invert=False, debounce=False),
            SwitchSettings(hw_switch=self.machine.switches["s_flipper_single_eos"].hw_switch, invert=False,
                           debounce=False),
            DriverSettings(hw_driver=self.machine.coils["c_flipper_single_main"].hw_driver,
                           pulse_settings=PulseSettings(power=1.0, duration=30),
                           hold_settings=HoldSettings(power=0.3), recycle=False),
            None    # this is not passed as we implement it in software
        )

        # nothing should happen on EOS close without flipper button active
        self.hit_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # nothing should happen on EOS open without flipper button active
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # flipper activation should also not yet do what
        self.hit_switch_and_run("s_flipper_single", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS opens. it should repulse
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_called_with(
            PulseSettings(power=1.0, duration=30), HoldSettings(power=0.3))
        self.machine.coils["c_flipper_single_main"].hw_driver.enable = MagicMock()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS closes only shortly
        self.hit_switch_and_run("s_flipper_single_eos", .1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS opens. it should not repulse yet
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS opens. it should repulse
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_called_with(
            PulseSettings(power=1.0, duration=30), HoldSettings(power=0.3))
        self.machine.coils["c_flipper_single_main"].hw_driver.enable = MagicMock()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # flipper button releases
        self.release_switch_and_run("s_flipper_single", 1)

        # eos opens - nothing should happen
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_called_with()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable = MagicMock()

        # disable flipper
        self.post_event("disable_flipper_single")
        self.assertPlaceholderEvaluates(False, "device.flippers.single_flipper.enabled")

        # nothing should happen on EOS close without flipper button active
        self.hit_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # nothing should happen on EOS open without flipper button active
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # flipper activation should also not yet do what
        self.hit_switch_and_run("s_flipper_single", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # EOS opens. nothing should happen as flipper is not enabled
        self.release_switch_and_run("s_flipper_single_eos", 1)
        self.machine.coils["c_flipper_single_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_single_main"].hw_driver.disable.assert_not_called()

        # release flipper prior to enabling the flipper
        self.release_switch_and_run("s_flipper_single", 1)

    def test_dual_wound_flipper_with_software_eos_repulse(self):
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse = MagicMock()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable = MagicMock()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable = MagicMock()

        # nothing should happen on EOS close without flipper button active
        self.hit_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # nothing should happen on EOS open without flipper button active
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # flipper activation should also not yet do what
        self.hit_switch_and_run("s_flipper_dual_wound", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS opens. nothing should happen as flipper is not enabled
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # release flipper prior to enabling the flipper
        self.release_switch_and_run("s_flipper_dual_wound", 1)

        self.machine.default_platform.set_pulse_on_hit_and_release_and_disable_rule = MagicMock()
        self.machine.default_platform.set_pulse_on_hit_and_enable_and_release_rule = MagicMock()
        self.assertPlaceholderEvaluates(False, "device.flippers.dual_wound_flipper.enabled")

        # enable flipper
        self.post_event("enable_flipper_dual_wound")
        self.assertPlaceholderEvaluates(True, "device.flippers.dual_wound_flipper.enabled")

        self.assertEqual(1, len(self.machine.default_platform.set_pulse_on_hit_and_release_and_disable_rule.
                                _mock_call_args_list))
        self.assertEqual(1, len(self.machine.default_platform.set_pulse_on_hit_and_enable_and_release_rule.
                                _mock_call_args_list))
        self.machine.default_platform.set_pulse_on_hit_and_release_and_disable_rule.assert_called_once_with(
            SwitchSettings(hw_switch=self.machine.switches["s_flipper_dual_wound"].hw_switch, invert=False,
                           debounce=False),
            SwitchSettings(hw_switch=self.machine.switches["s_flipper_dual_wound_eos"].hw_switch, invert=False,
                           debounce=False),
            DriverSettings(hw_driver=self.machine.coils["c_flipper_dual_wound_main"].hw_driver,
                           pulse_settings=PulseSettings(power=1.0, duration=30),
                           hold_settings=None, recycle=False),
            None    # this is not passed as we implement it in software
        )
        self.machine.default_platform.set_pulse_on_hit_and_enable_and_release_rule.assert_called_once_with(
            SwitchSettings(hw_switch=self.machine.switches["s_flipper_dual_wound"].hw_switch, invert=False,
                           debounce=False),
            DriverSettings(hw_driver=self.machine.coils["c_flipper_dual_wound_hold"].hw_driver,
                           pulse_settings=PulseSettings(power=1.0, duration=10),
                           hold_settings=HoldSettings(power=1.0), recycle=False)
        )

        # nothing should happen on EOS close without flipper button active
        self.hit_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # nothing should happen on EOS open without flipper button active
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # flipper activation should also not yet do what
        self.hit_switch_and_run("s_flipper_dual_wound", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS opens. it should repulse
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_called_with(
            PulseSettings(power=1.0, duration=30))
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse = MagicMock()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS closes only shortly
        self.hit_switch_and_run("s_flipper_dual_wound_eos", .1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS opens. it should not repulse yet
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS opens. it should repulse
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_called_with(
            PulseSettings(power=1.0, duration=30))
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse = MagicMock()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # flipper button releases
        self.release_switch_and_run("s_flipper_dual_wound", 1)

        # eos opens - nothing should happen
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_called_with()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable = MagicMock()

        # disable flipper
        self.post_event("disable_flipper_dual_wound")
        self.assertPlaceholderEvaluates(False, "device.flippers.dual_wound_flipper.enabled")

        # nothing should happen on EOS close without flipper button active
        self.hit_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # nothing should happen on EOS open without flipper button active
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # flipper activation should also not yet do what
        self.hit_switch_and_run("s_flipper_dual_wound", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS closes. nothing happens yet
        self.hit_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # EOS opens. nothing should happen as flipper is not enabled
        self.release_switch_and_run("s_flipper_dual_wound_eos", 1)
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.pulse.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.enable.assert_not_called()
        self.machine.coils["c_flipper_dual_wound_main"].hw_driver.disable.assert_not_called()

        # release flipper prior to enabling the flipper
        self.release_switch_and_run("s_flipper_dual_wound", 1)



"""Command which launches both the MPF core engine and MPF-MC."""

from importlib import import_module
import multiprocessing


def _start_mpf(mpf_path, machine_path, args):
    module = import_module('mpf.commands.game')
    module.Command(mpf_path, machine_path, args)


def _start_mc(mpf_path, machine_path, args):
    module = import_module('mpfmc.commands.mc')
    module.Command(mpf_path, machine_path, args + ["--both"])


class Command:

    """Command which runs game and mc."""

    def __init__(self, mpf_path, machine_path, args):
        """Run game and mc."""
        multiprocessing.set_start_method('spawn')
        mc = multiprocessing.Process(target=_start_mc,
                                     args=(mpf_path, machine_path, args))
        mc.start()
        _start_mpf(mpf_path, machine_path, args)
        mc.join()



import unittest

from mpf.core.rgba_color import RGBAColor

from mpf.core.rgb_color import RGBColor, RGBColorCorrectionProfile


class TestRGBColor(unittest.TestCase):

    def test_default_color(self):
        """Test the default color."""
        color = RGBColor()
        self.assertEqual((0, 0, 0), color.rgb)
        self.assertIn(color.name, ['black', 'off'])

    def test_rgba(self):
        # Test alpha channel."""
        color = RGBAColor("red")
        self.assertEqual((255, 0, 0), color.rgb)
        self.assertEqual((255, 0, 0, 255), color.rgba)
        self.assertEqual(255, color.opacity)
        self.assertEqual("red", color.name)

        color.rgba = (1, 2, 3, 4)
        self.assertEqual((1, 2, 3, 4), color.rgba)

        color = RGBAColor((255, 0, 0, 128))
        self.assertEqual((255, 0, 0), color.rgb)
        self.assertEqual((255, 0, 0, 128), color.rgba)
        self.assertEqual(128, color.opacity)
        self.assertEqual("red", color.name)

    def test_off_color(self):
        # Tests the 'Off' color (nicely readable in LED show files)
        color = RGBColor('Off')
        self.assertEqual((0, 0, 0), color.rgb)
        self.assertIn(color.name, ['black', 'off'])

    def test_static_conversion_utilities(self):
        # Tests initializing a color by name or hex string value
        # (names and hex values are not case-sensitive)
        self.assertEqual((240, 248, 255), RGBColor.string_to_rgb('aliceblue'))
        self.assertEqual((240, 248, 255), RGBColor.name_to_rgb('aliceblue'))
        self.assertEqual((240, 248, 255), RGBColor.hex_to_rgb('F0F8FF'))
        self.assertEqual((240, 248, 255), RGBColor.string_to_rgb('F0F8FF'))
        with self.assertRaises(AssertionError):
            RGBColor.string_to_rgb('non_existant')
        self.assertEqual((0, 0, 0), RGBColor.name_to_rgb('non_existant'))
        self.assertEqual((240, 248, 255), RGBColor.hex_to_rgb('f0f8ff'))
        self.assertEqual((240, 248, 255), RGBColor.string_to_rgb('f0f8ff'))

    def test_construction(self):
        self.assertEqual((240, 248, 255), RGBColor('aliceblue').rgb)
        self.assertEqual((240, 248, 255), RGBColor('F0F8FF').rgb)
        self.assertEqual((240, 248, 255), RGBColor('f0f8ff').rgb)
        self.assertEqual((240, 248, 255), RGBColor((240, 248, 255)).rgb)

    def test_properties(self):
        color1 = RGBColor('DarkSlateBlue')
        self.assertEqual((72, 61, 139), color1.rgb)
        self.assertEqual(72, color1.red)
        self.assertEqual(61, color1.green)
        self.assertEqual(139, color1.blue)
        self.assertEqual('darkslateblue', color1.name)
        self.assertEqual('483d8b', color1.hex)

        color2 = RGBColor((130, 130, 130))

        color_sum = color1 + color2
        self.assertEqual((202, 191, 255), color_sum.rgb)

        color_diff = color1 - color2
        self.assertEqual((0, 0, 9), color_diff.rgb)

        self.assertTrue(color1 != color2)
        self.assertFalse(color1 == color2)

        color_brightness = RGBColor("red%50")
        self.assertEqual((127, 0, 0), color_brightness.rgb)
        color_brightness = RGBColor("AABBCC%50")
        self.assertEqual((85, 93, 102), color_brightness.rgb)
        color_red = RGBColor("red")
        self.assertEqual((127, 0, 0), (color_red * 0.5).rgb)

    def test_color_blend(self):
        color1 = RGBColor((128, 64, 0))
        color2 = RGBColor((0, 32, 64))

        color_blend = RGBColor.blend(color1, color2, 0.25)
        self.assertEqual((96, 56, 16), color_blend.rgb)

        color_blend = RGBColor.blend(color1, color2, 0.5)
        self.assertEqual((64, 48, 32), color_blend.rgb)

        color_blend = RGBColor.blend(color1, color2, 0.75)
        self.assertEqual((32, 40, 48), color_blend.rgb)

    def test_color_correction(self):
        color = RGBColor(RGBColor.name_to_rgb('DarkGray'))
        self.assertEqual((169, 169, 169), color.rgb)

        # Tests default color correction profile (should be no correction)
        color_correction_profile = RGBColorCorrectionProfile("test")
        corrected_color = color_correction_profile.apply(color)
        self.assertEqual((169, 169, 169), corrected_color.rgb)

        # Test correction with default parameters
        color_correction_profile.generate_from_parameters()
        corrected_color = color_correction_profile.apply(color)
        self.assertEqual((91, 91, 91), corrected_color.rgb)

        # Test correction with new parameters
        color_correction_profile.generate_from_parameters(gamma=2.0,
                                                          whitepoint=(0.9, 0.85, 0.9),
                                                          linear_slope=0.75,
                                                          linear_cutoff=0.1)
        corrected_color = color_correction_profile.apply(color)
        self.assertEqual((77, 67, 77), corrected_color.rgb)

        # Test default correction profile
        default_profile = RGBColorCorrectionProfile.default()
        corrected_color = default_profile.apply(color)
        self.assertEqual((81, 81, 81), corrected_color.rgb)
        corrected_color = default_profile.apply(RGBColor((254, 254, 254)))
        self.assertEqual((252, 252, 252), corrected_color.rgb)

    def test_init_and_equal(self):
        black = RGBColor("black")
        color = RGBColor([1, 2, 3])
        self.assertEqual((1, 2, 3), color.rgb)
        color2 = RGBColor((1, 2, 3))
        color3 = RGBColor([1, 2, 3])
        color4 = RGBColor(color)
        color5 = RGBColor("010203")
        color6 = RGBColor("010203")
        color7 = RGBColor("")

        self.assertEqual(color2, color)
        self.assertEqual(color3, color)
        self.assertEqual(color4, color)
        self.assertEqual(color5, color)
        self.assertEqual(color6, color)
        self.assertEqual(color7, black)
        self.assertNotEqual(black, color)
        self.assertNotEqual(black, "010203")

        self.assertEqual("(1, 2, 3)", str(color5))

    def test_add_sub(self):
        color = RGBColor([1, 2, 3])
        color2 = color + (10, 10, 10)
        color3 = color + RGBColor((10, 10, 10))
        color4 = color2 - color3
        color5 = color2 - (11, 12, 13)
        self.assertEqual((11, 12, 13), color2.rgb)
        self.assertEqual((11, 12, 13), color3.rgb)
        self.assertEqual((0, 0, 0), color4.rgb)
        self.assertEqual((0, 0, 0), color5.rgb)



from mpf.tests.MpfTestCase import MpfTestCase
from unittest.mock import MagicMock


class TestDelay(MpfTestCase):

    def callback(self):
        pass

    def test_basic_functions(self):
        self.callback = MagicMock()
        self.assertEqual(len(self.machine.delay.delays), 0)

        # Create a one second delay
        self.machine.delay.add(1000, self.callback, "delay_test")
        self.assertEqual(len(self.machine.delay.delays), 1)
        self.assertIn("delay_test", self.machine.delay.delays.keys())

        # Advance 0.5 sec (callback should not have been called yet)
        self.advance_time_and_run(0.5)
        self.callback.assert_not_called()

        # Advance another 0.5 sec (callback should have been called)
        self.advance_time_and_run(0.5)
        self.callback.assert_called_with()
        self.assertEqual(len(self.machine.delay.delays), 0)
        self.assertNotIn("delay_test", self.machine.delay.delays.keys())

        # Create another one second delay
        self.callback.reset_mock()
        self.machine.delay.add(1000, self.callback, "delay_test2")
        self.assertEqual(len(self.machine.delay.delays), 1)
        self.assertIn("delay_test2", self.machine.delay.delays.keys())

        # Advance 0.5 sec (callback should not have been called yet)
        self.advance_time_and_run(0.5)
        self.callback.assert_not_called()

        # Now cancel the delay
        self.machine.delay.remove("delay_test2")
        self.assertEqual(len(self.machine.delay.delays), 0)
        self.assertNotIn("delay_test2", self.machine.delay.delays.keys())

        # Advance another 0.5 sec (callback should not be called since it was cancelled)
        self.advance_time_and_run(0.5)
        self.callback.assert_not_called()

    def test_remove(self):
        self.callback = MagicMock()
        self.machine.delay.add(1000, self.callback, "delay_test")

        self.assertTrue(self.machine.delay.check('delay_test'))
        self.machine.delay.remove('delay_test')
        self.advance_time_and_run(2)
        self.callback.assert_not_called()

    def test_check(self):
        self.callback = MagicMock()
        self.machine.delay.add(1000, self.callback, "delay_test")

        self.assertTrue(self.machine.delay.check('delay_test'))
        self.assertFalse(self.machine.delay.check('delay_test_fake'))

    def test_double_add(self):
        self.callback = MagicMock()
        # add delay
        self.machine.delay.add(100, self.callback, "delay_test")
        self.advance_time_and_run(.05)
        # add same name again. it should reset the time
        self.machine.delay.add(100, self.callback, "delay_test")
        self.advance_time_and_run(.06)
        self.callback.assert_not_called()
        self.advance_time_and_run(.05)
        self.callback.assert_any_call()

    def test_reset(self):
        self.callback = MagicMock()
        self.machine.delay.add(1000, self.callback, "delay_test")

        self.assertTrue(self.machine.delay.check('delay_test'))

        self.machine.delay.reset(2000, self.callback, "delay_test")

        self.advance_time_and_run(1.1)
        self.callback.assert_not_called()

        self.advance_time_and_run(1)
        self.callback.assert_any_call()

        # make sure reset works if there is no delay with that name
        self.machine.delay.reset(1000, self.callback, "delay_test2")
        self.advance_time_and_run(1.1)
        self.callback.assert_any_call()

    def test_clear(self):
        self.callback = MagicMock()
        self.machine.delay.add(1000, self.callback)
        self.machine.delay.add(2000, self.callback)

        self.machine.delay.clear()
        self.advance_time_and_run(3)

        self.callback.assert_not_called()

    def test_add_if_doesnt_exist(self):
        self.callback = MagicMock()
        self.machine.delay.add_if_doesnt_exist(1000, self.callback,
                                              "delay_test")

        self.advance_time_and_run(1.1)
        self.callback.assert_any_call()

        self.callback = MagicMock()
        self.machine.delay.add_if_doesnt_exist(1000, self.callback,
                                              "delay_test")
        self.machine.delay.add_if_doesnt_exist(500, self.callback,
                                      "delay_test")
        self.advance_time_and_run(.6)
        self.callback.assert_not_called()
        self.advance_time_and_run(.5)
        self.callback.assert_any_call()

    def test_run_now(self):
        self.callback = MagicMock()
        self.machine.delay.add(1000, self.callback, "delay_test")
        self.advance_time_and_run(.1)
        self.callback.assert_not_called()

        self.machine.delay.run_now("delay_test")
        self.advance_time_and_run(.1)
        self.callback.assert_any_call()

        self.callback = MagicMock()
        self.advance_time_and_run(1)
        self.callback.assert_not_called()



from mpf.tests.MpfTestCase import MpfTestCase, test_config


class TestSmartVirtualPlatform(MpfTestCase):

    def get_config_file(self):
        return 'test_smart_virtual.yaml'

    def get_machine_path(self):
        return 'tests/machine_files/smart_virtual_platform/'

    def get_platform(self):
        return 'smart_virtual'

    @test_config("test_entrance_switch.yaml")
    def test_entrance_switch(self):
        self.assertEqual(0, self.machine.ball_devices["trough"].balls)
        self.machine.default_platform.add_ball_to_device(self.machine.ball_devices["trough"])
        self.advance_time_and_run(1)
        self.assertEqual(1, self.machine.ball_devices["trough"].balls)

    @test_config("test_smart_virtual_initial.yaml")
    def test_eject(self):
        # device1_s1 is active in this test initially
        self.advance_time_and_run(.6)
        self.assertEqual(1, self.machine.ball_devices["device1"].balls)
        self.assertEqual(0, self.machine.ball_devices["device2"].balls)
        self.assertSwitchState("device2_s1", 0)
        self.assertSwitchState("device2_s2", 0)
        self.assertSwitchState("device1_s1", 1)
        self.assertSwitchState("device1_s2", 0)

        self.machine.ball_devices["device1"].eject()
        self.advance_time_and_run(1)

        self.assertEqual(0, self.machine.ball_devices["device1"].balls)
        self.assertEqual(1, self.machine.ball_devices["device2"].balls)

        self.assertSwitchState("device2_s1", 1)
        self.assertSwitchState("device2_s2", 0)
        self.assertSwitchState("device1_s1", 0)
        self.assertSwitchState("device1_s2", 0)

    @test_config("test_smart_virtual_initial.yaml")
    def test_eject_with_plunger(self):
        trough = self.machine.ball_devices["trough2"]
        plunger = self.machine.ball_devices["plunger2"]

        # add two balls to trough
        self.machine.default_platform.add_ball_to_device(trough)
        self.machine.default_platform.add_ball_to_device(trough)
        self.advance_time_and_run()

        self.assertEqual(2, trough.balls)
        self.assertEqual(0, plunger.balls)
        self.assertBallsOnPlayfield(0)
        self.assertSwitchState("trough2_1", 1)
        self.assertSwitchState("trough2_2", 1)
        self.assertSwitchState("trough2_3", 0)
        self.assertSwitchState("plunger2", 0)

        self.machine.playfield.add_ball(1, plunger)
        self.advance_time_and_run(1)

        self.assertEqual(1, trough.balls)
        self.assertEqual(1, plunger.balls)
        self.assertBallsOnPlayfield(0)
        self.assertSwitchState("trough2_1", 0)
        self.assertSwitchState("trough2_2", 1)
        self.assertSwitchState("trough2_3", 0)
        self.assertSwitchState("plunger2", 1)

        self.advance_time_and_run(3)
        self.assertEqual(1, trough.balls)
        self.assertEqual(0, plunger.balls)
        self.assertBallsOnPlayfield(0)
        self.assertSwitchState("trough2_1", 0)
        self.assertSwitchState("trough2_2", 1)
        self.assertSwitchState("trough2_3", 0)
        self.assertSwitchState("plunger2", 0)

        self.advance_time_and_run(10)
        self.assertEqual(1, trough.balls)
        self.assertEqual(0, plunger.balls)
        self.assertBallsOnPlayfield(1)
        self.assertSwitchState("trough2_1", 0)
        self.assertSwitchState("trough2_2", 1)
        self.assertSwitchState("trough2_3", 0)
        self.assertSwitchState("plunger2", 0)

    def _ball_swallower(self, unclaimed_balls, **kwargs):
        return {'unclaimed_balls': 0}

    def test_ball_device_with_entrance_switch(self):
        self.machine.events.add_handler('balldevice_device3_ball_enter',
                                        self._ball_swallower)

        self.assertEqual(0, self.machine.ball_devices["device3"].balls)
        self.advance_time_and_run(1)

        self.hit_and_release_switch('device3_s')
        self.advance_time_and_run()
        self.assertEqual(1, self.machine.ball_devices["device3"].balls)

        self.hit_and_release_switch('device3_s')
        self.advance_time_and_run()
        self.assertEqual(2, self.machine.ball_devices["device3"].balls)

        self.hit_and_release_switch('device3_s')
        self.advance_time_and_run()
        self.assertEqual(3, self.machine.ball_devices["device3"].balls)

        self.machine.ball_devices["device3"].eject()
        self.advance_time_and_run()
        self.assertEqual(2, self.machine.ball_devices["device3"].balls)

    def test_ball_device_with_entrance_switch_full_timeout(self):
        self.machine.events.add_handler('balldevice_device4_ball_enter',
                                        self._ball_swallower)

        self.assertEqual(0, self.machine.ball_devices["device4"].balls)
        self.advance_time_and_run(1)

        self.hit_and_release_switch('device4_s')
        self.advance_time_and_run()
        self.assertEqual(1, self.machine.ball_devices["device4"].balls)

        self.hit_and_release_switch('device4_s')
        self.advance_time_and_run()
        self.assertEqual(2, self.machine.ball_devices["device4"].balls)

        self.hit_switch_and_run('device4_s', 1)
        self.assertEqual(3, self.machine.ball_devices["device4"].balls)

        self.machine.playfield.add_ball(1, self.machine.ball_devices["device4"])
        self.advance_time_and_run(3)
        self.hit_switch_and_run('playfield', 1)

        self.assertEqual(2, self.machine.ball_devices["device4"].balls)
        self.assertSwitchState("device4_s", 0)

    def test_eject_with_no_ball(self):
        # tests that firing a coil of a device with no balls in it does not
        # put a ball in the target device.
        self.assertEqual(0, self.machine.ball_devices["device1"].balls)
        self.assertEqual(0, self.machine.ball_devices["device2"].balls)
        self.assertSwitchState("device2_s1", 0)
        self.assertSwitchState("device2_s2", 0)
        self.assertSwitchState("device1_s1", 0)
        self.assertSwitchState("device1_s2", 0)

        self.machine.coils["plunger"].pulse()
        self.advance_time_and_run(1)

        self.assertEqual(0, self.machine.ball_devices["device1"].balls)
        self.assertEqual(0, self.machine.ball_devices["device2"].balls)
        self.assertSwitchState("device2_s1", 0)
        self.assertSwitchState("device2_s2", 0)
        self.assertSwitchState("device1_s1", 0)
        self.assertSwitchState("device1_s2", 0)

    def test_start_active_switches(self):
        # tests that the virtual_platform_start_active_switches really do start
        # active.
        self.assertEqual(3, self.machine.ball_devices["trough"].balls)

    def test_drop_targets(self):
        # complete target
        self.assertFalse(self.machine.drop_targets['left3'].complete)
        self.hit_switch_and_run("switch3", 1)
        self.assertTrue(self.machine.drop_targets['left3'].complete)

        # reset target
        self.machine.drop_targets['left3'].reset()
        self.advance_time_and_run()
        self.assertSwitchState("switch3", 0)
        self.assertFalse(self.machine.drop_targets['left3'].complete)

        # knockdown single target
        self.machine.drop_targets['left3'].knockdown()
        self.advance_time_and_run()
        self.assertSwitchState("switch3", 1)
        self.assertTrue(self.machine.drop_targets['left3'].complete)

        # test bank
        self.assertFalse(self.machine.drop_targets['left1'].complete)
        self.assertFalse(self.machine.drop_targets['left2'].complete)
        self.assertFalse(self.machine.drop_target_banks['left_bank'].complete)

        self.hit_switch_and_run("switch1", .1)
        self.hit_switch_and_run("switch2", .1)
        self.assertTrue(self.machine.drop_targets['left1'].complete)
        self.assertTrue(self.machine.drop_targets['left2'].complete)
        self.assertTrue(self.machine.drop_target_banks['left_bank'].complete)

        # it should reset after 1.5s
        self.advance_time_and_run(1.5)
        self.assertSwitchState("switch1", 0)
        self.assertSwitchState("switch2", 0)
        self.assertFalse(self.machine.drop_targets['left1'].complete)
        self.assertFalse(self.machine.drop_targets['left2'].complete)
        self.assertFalse(self.machine.drop_target_banks['left_bank'].complete)

    @test_config("test_coil_fired_plunger.yaml")
    def test_coil_fired_plunger(self):
        self.advance_time_and_run(2)
        self.assertEqual(5, self.machine.ball_devices["trough"].balls)
        self.assertEqual(0, self.machine.ball_devices["shooter_lane"].balls)
        self.assertEqual(0, self.machine.playfield.balls)

        self.hit_and_release_switch('s_start')
        self.assertModeRunning('game')
        self.advance_time_and_run(3)

        self.assertEqual(4, self.machine.ball_devices["trough"].balls)
        self.assertEqual(1, self.machine.ball_devices["shooter_lane"].balls)
        self.assertEqual(0, self.machine.playfield.balls)

        self.hit_and_release_switch('s_shooter_lane')
        self.advance_time_and_run()

        self.assertEqual(4, self.machine.ball_devices["trough"].balls)
        self.assertEqual(0, self.machine.ball_devices["shooter_lane"].balls)
        self.assertEqual(0, self.machine.playfield.balls)

        self.hit_and_release_switch('s_standup')
        self.advance_time_and_run()

        self.assertEqual(4, self.machine.ball_devices["trough"].balls)
        self.assertEqual(0, self.machine.ball_devices["shooter_lane"].balls)
        self.assertEqual(1, self.machine.playfield.balls)



from unittest.mock import MagicMock
from mpf.tests.MpfTestCase import MpfTestCase


class TestMpfTestCase(MpfTestCase):

    def get_config_file(self):
        return 'test_mpftestcase.yaml'

    def get_machine_path(self):
        return 'tests/machine_files/mpftestcase/'

    def get_platform(self):
        return 'smart_virtual'

    def _interval(self):
        self.counter += 1

    def test_schedule_interval_in_tests(self):
        self.counter = 0
        self.machine.clock.schedule_interval(self._interval, 1)

        self.advance_time_and_run(1.1)
        self.assertEqual(1, self.counter)
        for i in range(10):
            self.advance_time_and_run(.1)
        self.assertEqual(2, self.counter)

    def test_test_case(self):
        # test the delay
        self.delay_callback = MagicMock()
        self.machine.default_platform.delay.add(ms=500,
                                                callback=self.delay_callback)

        self.advance_time_and_run(.25)
        self.delay_callback.assert_not_called()

        self.advance_time_and_run(1)
        self.delay_callback.assert_called_once_with()

        # test the timed switch event
        self.assertIn('switch1', self.machine.switches)
        self.switch_callback = MagicMock()
        self.machine.switch_controller.add_switch_handler('switch1', self.switch_callback, ms=1000)

        self.machine.switch_controller.process_switch('switch1', 1)
        self.advance_time_and_run(.5)
        self.switch_callback.assert_not_called()

        self.advance_time_and_run(1)
        self.switch_callback.assert_called_once_with()

    def test_multiples(self):
        # test multiples and jump ahead to see if they were called
        self.id_list = list()

        self.machine.default_platform.delay.add(ms=500,
                                                callback=self._callback,
                                                _id='d1')
        self.machine.default_platform.delay.add(ms=1500,
                                                callback=self._callback,
                                                _id='d2')
        self.machine.default_platform.delay.add(ms=2500,
                                                callback=self._callback,
                                                _id='d3')

        self.machine.switch_controller.add_switch_handler(
            'switch1', self._callback, ms=1000, callback_kwargs={'_id': 's1'})
        self.machine.switch_controller.add_switch_handler(
            'switch1', self._callback, ms=2000, callback_kwargs={'_id': 's2'})
        self.machine.switch_controller.add_switch_handler(
            'switch1', self._callback, ms=3000, callback_kwargs={'_id': 's3'})

        self.machine.switch_controller.process_switch('switch1', 1)
        self.advance_time_and_run(10)

        # make sure the callback was called in the right order
        self.assertEqual(self.id_list[0][0], 'd1')
        self.assertEqual(self.id_list[1][0], 's1')
        self.assertEqual(self.id_list[2][0], 'd2')
        self.assertEqual(self.id_list[3][0], 's2')
        self.assertEqual(self.id_list[4][0], 'd3')
        self.assertEqual(self.id_list[5][0], 's3')

        # make sure they were all called about 500ms apart
        self.assertAlmostEqual(self.id_list[1][1] - self.id_list[0][1], .5,
                               delta=0.01)
        self.assertAlmostEqual(self.id_list[2][1] - self.id_list[1][1], .5,
                               delta=0.01)
        self.assertAlmostEqual(self.id_list[3][1] - self.id_list[2][1], .5,
                               delta=0.01)
        self.assertAlmostEqual(self.id_list[4][1] - self.id_list[3][1], .5,
                               delta=0.01)
        self.assertAlmostEqual(self.id_list[5][1] - self.id_list[4][1], .5,
                               delta=0.01)

    def _callback(self, _id):
        # print(self.id_list)
        self.id_list.append((_id, self.machine.clock.get_time()))



"""Interface for an accelerometer device."""
from typing import List

import abc


class AccelerometerPlatformInterface(metaclass=abc.ABCMeta):

    """Interface for accelerometer devices in platforms.

    Currently no public methods.
    """

    __slots__ = []  # type: List[str]
